@@ -30,6 +30,8 @@ typedef struct _ProcData ProcData;
 #include <gconf/gconf-client.h>
 #include <glibtop/cpu.h>
 
+#include <time.h>
+
 #include "smooth_refresh.h"
 
 #include "load-graph.h"
@@ -41,8 +43,6 @@ enum
 	ACTIVE_PROCESSES
 };
 
-#define NCPUSTATES 4
-
 enum
 {
 	MIN_UPDATE_INTERVAL = 1 * 1000,
@@ -57,7 +57,6 @@ struct _ProcConfig
         gboolean	show_kill_warning;
         gboolean	show_hide_message;
         gboolean	show_tree;
-        gboolean	show_threads;
 	gboolean	show_all_fs;
 	guint		update_interval;
  	gint		graph_update_interval;
@@ -90,23 +89,24 @@ struct _ProcInfo
 	gchar		*status; /* shared, don't free it ! */
 	gchar		*security_context;
 
+	time_t		start_time;
 	guint64		cpu_time_last;
 
 	guint64		vmsize;
 	guint64		memres;
 	guint64		memwritable;
 	guint64		memshared;
+	guint64		mem; /* estimated memory usage */
 	unsigned long	memxserver;
 
 	guint		pcpu; /* 0% - 100% */
 	gint		nice;
 
 	guint		pid;
-	gint		uid;
+	guint		uid;
 
 	guint		is_visible	: 1;
 	guint		is_running	: 1;
-	guint		is_thread	: 1;
 	guint		is_blacklisted	: 1;
 };
 
