@@ -29,6 +29,7 @@
 #include <glibtop/proclist.h>
 #include <glibtop/procstate.h>
 #include <glibtop/procmem.h>
+#include <glibtop/procmap.h>
 #include <glibtop/proctime.h>
 #include <glibtop/procuid.h>
 #include <glibtop/procargs.h>
@@ -38,6 +39,7 @@
 #include <libwnck/libwnck.h>
 #include <sys/stat.h>
 #include <pwd.h>
+#include <time.h>
 
 #include <libgnomevfs/gnome-vfs-utils.h>
 
@@ -46,10 +48,10 @@
 #include "callbacks.h"
 #include "prettytable.h"
 #include "util.h"
-#include "infoview.h"
 #include "interface.h"
 #include "favorites.h"
 #include "selinux.h"
+#include "e_date.h"
 
 
 static guint64 total_time = 1;
@@ -64,34 +66,38 @@ sort_ints (GtkTreeModel *model, GtkTreeIter *itera, GtkTreeIter *iterb, gpointer
 
 	gtk_tree_model_get (model, itera, COL_POINTER, &infoa, -1);
 	gtk_tree_model_get (model, iterb, COL_POINTER, &infob, -1);
-	g_return_val_if_fail (infoa, 0);
-	g_return_val_if_fail (infob, 0);
+
+	g_assert(infoa);
+	g_assert(infob);
 
 	switch (col) {
-	case COL_MEM:
-		return PROCMAN_RCMP(infoa->mem, infob->mem);
-
 	case COL_VMSIZE:
 		return PROCMAN_RCMP(infoa->vmsize, infob->vmsize);
 
 	case COL_MEMRES:
 		return PROCMAN_RCMP(infoa->memres, infob->memres);
 
+	case COL_MEMWRITABLE:
+		return PROCMAN_RCMP(infoa->memwritable, infob->memwritable);
+
 	case COL_MEMSHARED:
 		return PROCMAN_RCMP(infoa->memshared, infob->memshared);
 
-	case COL_MEMRSS:
-		return PROCMAN_RCMP(infoa->memrss, infob->memrss);
-
 	case COL_MEMXSERVER:
 		return PROCMAN_RCMP(infoa->memxserver, infob->memxserver);
 
 	case COL_CPU_TIME:
 		return PROCMAN_RCMP(infoa->cpu_time_last, infob->cpu_time_last);
 
+	case COL_START_TIME:
+		return PROCMAN_CMP(infoa->start_time, infob->start_time);
+
 	case COL_CPU:
 		return PROCMAN_RCMP(infoa->pcpu, infob->pcpu);
 
+	case COL_MEM:
+		return PROCMAN_RCMP(infoa->mem, infob->mem);
+
 	default:
 		g_assert_not_reached();
 		return 0;
@@ -100,6 +106,48 @@ sort_ints (GtkTreeModel *model, GtkTreeIter *itera, GtkTreeIter *iterb, gpointer
 
 
 
+static GtkWidget *
+create_proctree(GtkTreeModel *model)
+{
+	GtkWidget *proctree = NULL;
+	GModule *sexy;
+
+	sexy = g_module_open("libsexy.so",
+			     G_MODULE_BIND_LAZY | G_MODULE_BIND_LOCAL);
+
+	if (sexy) {
+		GtkWidget* (*new)(void);
+		void (*set_column)(void*, guint);
+
+		if (g_module_symbol(sexy,
+				    "sexy_tree_view_new",
+				    &new) &&
+		    g_module_symbol(sexy,
+				    "sexy_tree_view_set_tooltip_label_column",
+				    &set_column)) {
+
+			g_module_make_resident(sexy);
+			g_print("Found libsexy\n");
+
+			proctree = new();
+			gtk_tree_view_set_model(GTK_TREE_VIEW(proctree), model);
+			set_column(proctree, COL_ARGS);
+		} else {
+			g_module_close(sexy);
+		}
+	}
+
+	if (!proctree) {
+		proctree = gtk_tree_view_new_with_model(model);
+	}
+
+	return proctree;
+}
+
+
+
+
+
 static void
 set_proctree_reorderable(ProcData *procdata)
 {
@@ -238,29 +286,30 @@ proctable_new (ProcData * const procdata)
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *cell_renderer;
 
-	static const gchar *titles[] = {
+	const gchar *titles[] = {
 		N_("Process Name"),
 		N_("User"),
 		N_("Status"),
-		N_("Memory"),
 		N_("Virtual Memory"),
 		N_("Resident Memory"),
+		N_("Writable Memory"),
 		N_("Shared Memory"),
-		N_("RSS Memory"),
 		N_("X Server Memory"),
 		/* xgettext:no-c-format */ N_("% CPU"),
-		N_("CPU time"),
+		N_("CPU Time"),
+		N_("Started"),
 		N_("Nice"),
 		N_("ID"),
 		N_("Security Context"),
 		N_("Arguments"),
+		N_("Memory"),
 		NULL,
 		"POINTER"
 	};
 
-	gint i;
+	g_assert(COL_MEM == 15);
 
-	PROCMAN_GETTEXT_ARRAY_INIT(titles);
+	gint i;
 
 	scrolled = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
@@ -271,23 +320,24 @@ proctable_new (ProcData * const procdata)
 				    G_TYPE_STRING,	/* Process Name */
 				    G_TYPE_STRING,	/* User		*/
 				    G_TYPE_STRING,	/* Status	*/
-				    G_TYPE_STRING,      /* Memory	*/
 				    G_TYPE_STRING,	/* VM Size	*/
 				    G_TYPE_STRING,	/* Resident Memory */
+				    G_TYPE_STRING,	/* Writable Memory */
 				    G_TYPE_STRING,	/* Shared Memory */
-				    G_TYPE_STRING,	/* RSS Memory	*/
 				    G_TYPE_STRING,	/* X Server Memory */
 				    G_TYPE_UINT,	/* % CPU	*/
 				    G_TYPE_STRING,	/* CPU time	*/
+				    G_TYPE_STRING,	/* Started	*/
 				    G_TYPE_INT,		/* Nice		*/
 				    G_TYPE_UINT,	/* ID		*/
 				    G_TYPE_STRING,	/* Security Context */
 				    G_TYPE_STRING,	/* Arguments	*/
+				    G_TYPE_STRING,	/* Memory       */
 				    GDK_TYPE_PIXBUF,	/* Icon		*/
 				    G_TYPE_POINTER	/* ProcInfo	*/
 		);
 
-	proctree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
+	proctree = create_proctree(GTK_TREE_MODEL(model));
 	gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (proctree),
 					     search_equal_func,
 					     NULL,
@@ -311,29 +361,30 @@ proctable_new (ProcData * const procdata)
 	gtk_tree_view_column_set_attributes (column, cell_renderer,
 					     "text", COL_NAME,
 					     NULL);
-	gtk_tree_view_column_set_title (column, titles[0]);
+	gtk_tree_view_column_set_title (column, _(titles[0]));
 	gtk_tree_view_column_set_sort_column_id (column, COL_NAME);
 	gtk_tree_view_column_set_resizable (column, TRUE);
+	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_min_width (column, 1);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (proctree), column);
 	gtk_tree_view_set_expander_column (GTK_TREE_VIEW (proctree), column);
 
 
-	for (i = COL_USER; i <= COL_ARGS; i++) {
+	for (i = COL_USER; i <= COL_MEM; i++) {
 		cell_renderer = gtk_cell_renderer_text_new ();
 
 		switch(i)
 		{
-		case COL_MEM:
 		case COL_VMSIZE:
 		case COL_MEMRES:
+		case COL_MEMWRITABLE:
 		case COL_MEMSHARED:
-		case COL_MEMRSS:
 		case COL_MEMXSERVER:
 		case COL_CPU:
 		case COL_NICE:
 		case COL_PID:
 		case COL_CPU_TIME:
+		case COL_MEM:
 			g_object_set(G_OBJECT(cell_renderer),
 				     "xalign", 1.0f,
 				     NULL);
@@ -344,9 +395,16 @@ proctable_new (ProcData * const procdata)
 		gtk_tree_view_column_set_attributes (column, cell_renderer,
 						     "text", i,
 						     NULL);
-		gtk_tree_view_column_set_title (column, titles[i]);
+		gtk_tree_view_column_set_title (column, _(titles[i]));
 		gtk_tree_view_column_set_sort_column_id (column, i);
 		gtk_tree_view_column_set_resizable (column, TRUE);
+
+		switch (i) {
+		case COL_ARGS:
+			gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
+			break;
+		}
+
 		gtk_tree_view_column_set_min_width (column, 1);
 		gtk_tree_view_append_column (GTK_TREE_VIEW (proctree), column);
 	}
@@ -354,18 +412,19 @@ proctable_new (ProcData * const procdata)
 	gtk_container_add (GTK_CONTAINER (scrolled), proctree);
 
 
-	for(i = COL_NAME; i <= COL_ARGS; i++)
+	for(i = COL_NAME; i <= COL_MEM; i++)
 	{
 		switch(i)
 		{
-		case COL_MEM:
 		case COL_VMSIZE:
 		case COL_MEMRES:
+		case COL_MEMWRITABLE:
 		case COL_MEMSHARED:
-		case COL_MEMRSS:
 		case COL_MEMXSERVER:
 		case COL_CPU_TIME:
+		case COL_START_TIME:
 		case COL_CPU:
+		case COL_MEM:
 			gtk_tree_sortable_set_sort_func (
 				GTK_TREE_SORTABLE (model),
 				i,
@@ -404,9 +463,9 @@ proctable_new (ProcData * const procdata)
 
 
 static void
-proctable_free_info (ProcData *procdata, ProcInfo *info)
+proctable_free_info(ProcInfo *info)
 {
-	g_return_if_fail(info != NULL);
+	g_assert(info != NULL);
 
 	if(info->pixbuf) {
 		g_object_unref (info->pixbuf);
@@ -416,7 +475,7 @@ proctable_free_info (ProcData *procdata, ProcInfo *info)
 	g_free (info->arguments);
 	g_free (info->security_context);
 	g_slist_free (info->children);
-	g_mem_chunk_free (procdata->procinfo_allocator, info);
+	g_slice_free(ProcInfo, info);
 }
 
 
@@ -503,6 +562,29 @@ get_process_user(ProcData* procdata, ProcInfo* info, uid_t uid)
 
 
 
+static void get_process_memory_writable(ProcInfo *info)
+{
+	glibtop_proc_map buf;
+	glibtop_map_entry *maps;
+	unsigned i;
+
+	info->memwritable = 0;
+
+	maps = glibtop_get_proc_map(&buf, info->pid);
+
+	for (i = 0; i < buf.number; ++i) {
+#ifdef __linux__
+		info->memwritable += maps[i].private_dirty;
+#else
+		if (maps[i].perm & GLIBTOP_MAP_PERM_WRITE)
+			info->memwritable += maps[i].size;
+#endif
+	}
+
+	g_free(maps);
+}
+
+
 static void
 get_process_memory_info(ProcInfo *info)
 {
@@ -515,15 +597,19 @@ get_process_memory_info(ProcInfo *info)
 
 	glibtop_get_proc_mem(&procmem, info->pid);
 
-	info->mem	= procmem.size;
 	info->vmsize	= procmem.vsize;
 	info->memres	= procmem.resident;
 	info->memshared	= procmem.share;
-	info->memrss	= procmem.rss;
 
 	info->memxserver = xresources.total_bytes_estimate;
+
+	get_process_memory_writable(info);
+
+	info->mem = info->memxserver + info->memwritable;
 }
 
+
+
 ProcInfo *
 proctable_find_process (guint pid, ProcData *procdata)
 {
@@ -542,7 +628,6 @@ find_parent (ProcData *procdata, guint pid)
 }
 
 
-
 static char *
 format_duration_for_display (double d)
 {
@@ -571,39 +656,47 @@ format_duration_for_display (double d)
 static void
 update_info_mutable_cols(GtkTreeStore *store, ProcData *procdata, ProcInfo *info)
 {
-	gchar *mem, *vmsize, *memres, *memshared, *memrss, *memxserver, *cpu_time;
+	gchar *vmsize, *memres, *memwritable, *memshared, *memxserver,
+		*cpu_time, *start_time, *mem;
 
-	mem	   = SI_gnome_vfs_format_file_size_for_display (info->mem);
 	vmsize	   = SI_gnome_vfs_format_file_size_for_display (info->vmsize);
 	memres	   = SI_gnome_vfs_format_file_size_for_display (info->memres);
+	memwritable = SI_gnome_vfs_format_file_size_for_display (info->memwritable);
 	memshared  = SI_gnome_vfs_format_file_size_for_display (info->memshared);
-	memrss	   = SI_gnome_vfs_format_file_size_for_display (info->memrss);
 	memxserver = SI_gnome_vfs_format_file_size_for_display (info->memxserver);
+	mem = SI_gnome_vfs_format_file_size_for_display(info->mem);
 
 	cpu_time = format_duration_for_display (info->cpu_time_last / procdata->frequency);
 
+	/* FIXME: does it worths it to display relative to $now date ?
+	   absolute date wouldn't required to be updated on every refresh */
+	start_time = procman_format_date_for_display(info->start_time);
+
 	gtk_tree_store_set (store, &info->node,
 			    COL_STATUS, info->status,
 			    COL_USER, info->user,
-			    COL_MEM, mem,
 			    COL_VMSIZE, vmsize,
 			    COL_MEMRES, memres,
+			    COL_MEMWRITABLE, memwritable,
 			    COL_MEMSHARED, memshared,
-			    COL_MEMRSS, memrss,
 			    COL_MEMXSERVER, memxserver,
 			    COL_CPU, info->pcpu,
 			    COL_CPU_TIME, cpu_time,
+			    COL_START_TIME, start_time,
 			    COL_NICE, info->nice,
+			    COL_MEM, mem,
 			    -1);
 
-	/* We don't bother updating COL_SECURITYCONTEXT as it can never change */
-	g_free (mem);
+	/* FIXME: We don't bother updating COL_SECURITYCONTEXT as it can never change.
+	   even on fork ? */
 	g_free (vmsize);
 	g_free (memres);
+	g_free (memwritable);
 	g_free (memshared);
-	g_free (memrss);
 	g_free (memxserver);
 	g_free (cpu_time);
+	g_free(start_time);
+	g_free(mem);
 }
 
 
@@ -612,7 +705,6 @@ void
 insert_info_to_tree (ProcInfo *info, ProcData *procdata)
 {
 	GtkTreeModel *model;
-	gchar *name;
 
 	/* Don't show process if it is not running */
 	if ((procdata->config.whose_process == ACTIVE_PROCESSES) &&
@@ -627,14 +719,6 @@ insert_info_to_tree (ProcInfo *info, ProcData *procdata)
 	}
 	info->is_blacklisted = FALSE;
 
-	/* Don't show threads */
-	if (!procdata->config.show_threads && info->is_thread)
-		return;
-	else if (info->is_thread)
-		name = g_strdup_printf(_("%s (thread)"), info->name);
-	else
-		name = g_strdup (info->name);
-
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (procdata->tree));
 	if (info->parent && procdata->config.show_tree && info->parent->is_visible) {
 		GtkTreePath *parent_node = gtk_tree_model_get_path (model, &info->parent->node);
@@ -658,7 +742,7 @@ insert_info_to_tree (ProcInfo *info, ProcData *procdata)
 	gtk_tree_store_set (GTK_TREE_STORE (model), &info->node,
 			    COL_POINTER, info,
 			    COL_PIXBUF, info->pixbuf,
-			    COL_NAME, name,
+			    COL_NAME, info->name,
 			    COL_ARGS, info->arguments,
 			    COL_PID, info->pid,
 			    COL_SECURITYCONTEXT, info->security_context,
@@ -667,8 +751,6 @@ insert_info_to_tree (ProcInfo *info, ProcData *procdata)
 	update_info_mutable_cols(GTK_TREE_STORE (model), procdata, info);
 
 	info->is_visible = TRUE;
-
-	g_free(name);
 }
 
 
@@ -702,7 +784,7 @@ remove_info_from_tree (ProcInfo *info, ProcData *procdata)
 {
 	GtkTreeModel *model;
 
-	g_return_if_fail (info);
+	g_assert(info);
 
 	if (!info->is_visible)
 		return;
@@ -743,7 +825,7 @@ remove_info_from_list (ProcInfo *info, ProcData *procdata)
 	/* Remove from list */
 	procdata->info = g_list_remove (procdata->info, info);
 	g_hash_table_remove(procdata->pids, GINT_TO_POINTER(info->pid));
-	proctable_free_info (procdata, info);
+	proctable_free_info(info);
 }
 
 
@@ -797,7 +879,7 @@ update_info (ProcData *procdata, ProcInfo *info)
 
 
 static ProcInfo *
-get_info (ProcData *procdata, gint pid)
+procinfo_new (ProcData *procdata, gint pid)
 {
 	ProcInfo *info;
 	glibtop_proc_state procstate;
@@ -806,7 +888,7 @@ get_info (ProcData *procdata, gint pid)
 	glibtop_proc_args procargs;
 	gchar** arguments;
 
-	info = g_chunk_new0(ProcInfo, procdata->procinfo_allocator);
+	info = g_slice_new0(ProcInfo);
 
 	info->pid = pid;
 	info->uid = -1;
@@ -816,6 +898,7 @@ get_info (ProcData *procdata, gint pid)
 	glibtop_get_proc_time (&proctime, pid);
 	arguments = glibtop_get_proc_argv (&procargs, pid, 0);
 
+	/* FIXME : wrong. name and arguments may change with exec* */
 	get_process_name (info, procstate.cmd, arguments[0]);
 	get_process_user (procdata, info, procstate.uid);
 
@@ -824,6 +907,7 @@ get_info (ProcData *procdata, gint pid)
 
 	info->pcpu = 0;
 	info->cpu_time_last = proctime.rtime;
+	info->start_time = proctime.start_time;
 	info->nice = procuid.nice;
 
 	get_process_memory_info(info);
@@ -835,9 +919,6 @@ get_info (ProcData *procdata, gint pid)
 	info->parent = find_parent (procdata, procuid.ppid);
 	if (info->parent) {
 		info->parent->children = g_slist_prepend (info->parent->children, info);
-		if(g_str_equal(info->name, info->parent->name)
-		   && info->parent->mem == info->mem)
-			info->is_thread = TRUE;
 	}
 
 	info->is_visible = FALSE;
@@ -869,7 +950,7 @@ refresh_list (ProcData *procdata, const unsigned *pid_list, const guint n)
 
 		info = proctable_find_process (pid_list[i], procdata);
 		if (!info) {
-			info = get_info (procdata, pid_list[i]);
+			info = procinfo_new (procdata, pid_list[i]);
 			g_ptr_array_add(addition_list, info);
 			procdata->info = g_list_append (procdata->info, info);
 			g_hash_table_insert(procdata->pids, GINT_TO_POINTER(info->pid), info);
@@ -969,9 +1050,6 @@ proctable_update_all (ProcData * const procdata)
 	g_free (string);
 
 	proctable_update_list (procdata);
-
-	if (procdata->config.show_more_info)
-		infoview_update (procdata);
 }
 
 
@@ -998,7 +1076,7 @@ proctable_free_table (ProcData * const procdata)
 	while (list)
 	{
 		ProcInfo *info = list->data;
-		proctable_free_info (procdata, info);
+		proctable_free_info(info);
 		list = g_list_next (list);
 	}
 
